
from django import forms
from django.contrib import admin

from request_signer.models import AuthorizedClient

class AuthorizedClientForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(AuthorizedClientForm, self).__init__(*args, **kwargs)
        self.fields['private_key'].widget.attrs['readonly'] = 'readonly'
        self.fields['private_key'].widget.attrs['style'] = 'width: 300px; background-color: #ccc;'

    class Meta(object):
        model = AuthorizedClient

class AuthorizedClientAdmin(admin.ModelAdmin):
    list_display = ['client_id', 'is_active', 'created', 'updated']
    fields = ['client_id', 'private_key', 'is_active']
    form = AuthorizedClientForm

admin.site.register(AuthorizedClient, AuthorizedClientAdmin)



from generic_request_signer.client import Client
from generic_request_signer.exceptions import WebException, HttpMethodNotAllowed
from generic_request_signer.request import Request
from generic_request_signer.response import Response
from generic_request_signer.factory import SignedRequestFactory



from request_signer.client.generic import Client, django_backend


class DjangoClient(Client):

    def __init__(self):
        api_credentials = django_backend.DjangoSettingsApiCredentialsBackend(self)
        super(DjangoClient, self).__init__(api_credentials)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'AuthorizedClient'
        db.create_table('request_signer_authorizedclient', (
            ('client_id', self.gf('django.db.models.fields.CharField')(max_length=20, primary_key=True)),
            ('private_key', self.gf('django.db.models.fields.CharField')(default='ZTWXjy01_v-muyWoLMSYOteEniKiPuqVim1ah_dfd2A=', max_length=100)),
            ('is_active', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('updated', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal('request_signer', ['AuthorizedClient'])


    def backwards(self, orm):
        
        # Deleting model 'AuthorizedClient'
        db.delete_table('request_signer_authorizedclient')


    models = {
        'request_signer.authorizedclient': {
            'Meta': {'object_name': 'AuthorizedClient'},
            'client_id': ('django.db.models.fields.CharField', [], {'max_length': '20', 'primary_key': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'private_key': ('django.db.models.fields.CharField', [], {'default': "'jAqUpViHFDNVg0IhGSaXxsi1RrhO9hHpmvUuiRr0q6I='", 'max_length': '100'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['request_signer']



from cStringIO import StringIO

from lettuce import before, after, world
from django.db import connection
from django.core.management import call_command
from south.management.commands import patch_for_test_db_setup

@before.all
def setup_test_database():
    patch_for_test_db_setup()
    world.test_database_name = connection.creation.create_test_db(verbosity=0, autoclobber=True)

@before.each_scenario
def clean_db(scenario):
    call_command('flush', interactive=False, stderr=StringIO())

@after.all
def teardown_test_database(scenario):
    connection.creation.destroy_test_db(world.test_database_name, verbosity=0)



from django import http
from django.views.generic.base import View

class SampleView(View):

    def get(self, request):
        return http.HttpResponse("hi")

    def post(self, request):
        return http.HttpResponse("hi")



from request_signer.client.generic import Client, WebException, django_backend


__all__ = (
    "BaseDjangoRestClient",
)


class BaseDjangoRestClient(Client):
    """
    Base Client for working with an api server using djangorestframework.

    Assumes your url structure of the api is the following:
      /api/endpoint/<item_list_key>/
      /api/endpoint/<item_list_key>/<item_detail_key>/

      Example:
      /api/endpoint/<company>/             # returns list of items for the company
      /api/endpoint/<company>/<service>/   # returns single item for company


    Django doesn't handle post data properly when the request is a "PUT", or
    anything other than "POST" really. So, for anything other than a GET or POST
    we need to add a "_method=PUT" or equivalent, which is how the django rest framework
    gets around this issue, but still uses full rest methods.
    """


    def __init__(self, api_credentials=None):
        api_credentials = api_credentials or django_backend.DjangoSettingsApiCredentialsBackend(self)
        super(BaseDjangoRestClient, self).__init__(api_credentials)

    BASE_API_ENDPOINT = None

    def build_endpoint(self, group_key, item_key=None):
        endpoint = "{base}{group_key}/".format(base=self.BASE_API_ENDPOINT, group_key=group_key)
        if item_key:
            endpoint += "{item_key}/".format(item_key=item_key)
        return endpoint

    def _get_json_response(self, http_method, endpoint, data=None):
        headers = {"Accept": "application/json"}
        return self._get_response(http_method, endpoint, data, headers=headers)

    def get_list(self, group_key):
        """
        :param group_key:
            The key to the group of items desired (eg. company_id)

        :returns:
            Returns list of items returned.
            When client returns a 404 returned from the client,
            an empty list is returned instead of an exception
        """
        endpoint = self.build_endpoint(group_key)
        r = self._get_json_response("GET", endpoint)
        if not r.is_successful and r.status_code != 404:
            raise WebException(r.read())
        return r.json

    def get_item(self, group_key, item_key):
        """
        :param group_key:
            The key to the group of items desired (eg. company_id)
        :param item_key:
            The key to the item desired

        :returns:
            Returns dictionary representation of item.
        """
        endpoint = self.build_endpoint(group_key, item_key)
        r = self._get_json_response("GET", endpoint)
        if not r.is_successful and r.status_code != 404:
            raise WebException(r.read())
        return r.json

    def create(self, group_key, **attrs):
        """
        :param group_key:
            The key to the group of items desired (eg. company_id)
        :param attrs:
            Attributes used to create the item.
            Note: the keys of "group key" and "item key" will need to be include
                  in the attrs or the item won't get built properly.
        :returns:
            JSON representation of item on success, raises exception on error
        """
        endpoint = self.build_endpoint(group_key)
        r = self._get_json_response("POST", endpoint, data=attrs)
        if not r.is_successful:
            raise WebException(r.read())
        return r.json

    def update(self, group_key, item_key, **attrs):
        """
        :param group_key:
            The key to the group of items desired (eg. company_id)
        :param item_key:
            The key to the item desired
        :param attrs:
            Attributes used to create the item.
            Note: the keys of "group key" and "item key" will need to be include
                  in the attrs or the item won't get built properly.
        :returns:
            JSON representation of item on success, raises exception on error
        """
        endpoint = self.build_endpoint(group_key, item_key)
        attrs["_method"] = "PUT"
        r = self._get_json_response("POST", endpoint, data=attrs)
        if not r.is_successful:
            raise WebException(r.read())
        return r.json

    def delete(self, group_key, item_key):
        """
        :param group_key:
            The key to the group of items desired (eg. company_id)
        :param item_key:
            The key to the item desired
        :returns:
            JSON response on success (should be empty dict)
            raises exception on error, but 404 means resource doesn't
            exist so there is nothing to delete and we let it slide.
        """
        endpoint = self.build_endpoint(group_key, item_key)
        attrs = {"_method": "DELETE"}
        r = self._get_json_response("POST", endpoint, data=attrs)
        if not r.is_successful and r.status_code != 404:
            raise WebException(r.read())
        return r.json




from optparse import make_option

from django.core.management import BaseCommand, CommandError
from request_signer.models import AuthorizedClient, create_private_key

class Command(BaseCommand):
    help = "Creates an authorized client. You can provide the exact private key"\
           "to use, or the command will auto generate one if not provided."

    option_list = BaseCommand.option_list + (
        make_option("-c", "--client", dest="client", action="store"),
        make_option("-k", "--key", dest="key", action="store"),
    )

    def handle(self, *args, **options):
        client = options.get("client")
        key = options.get("key")
        if not client:
            raise CommandError("Client is required.")

        if not key:
            key = create_private_key()

        client = AuthorizedClient.objects.create(client_id=client, private_key=key)
        self.stdout.write("\nYour client id is: '{0}'\n".format(client.client_id))
        self.stdout.write("Your private key is: '{0}'\n".format(client.private_key))







import base64
import hashlib
import hmac
import random
import time

from django.db import models

def create_private_key():
    """
    Creates new private key to be used for signing requests.

    :returns:
        base64_urlsafe private key
    """
    msg = "{0}{1}".format(time.time(), random.random())
    s = hmac.new("request-signer", msg, hashlib.sha256)
    return base64.urlsafe_b64encode(s.digest())


class AuthorizedClient(models.Model):
    client_id = models.CharField(max_length=20, primary_key=True)
    private_key = models.CharField(max_length=100, default=create_private_key)
    is_active = models.BooleanField(default=True)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    @classmethod
    def get_by_client(cls, client_id):
        try:
            return cls.objects.get(pk=client_id, is_active=True)
        except cls.DoesNotExist:
            return None


class AuthorizedServiceProvider(models.Model):
    base_url = models.URLField(primary_key=True)
    client_id = models.CharField(max_length=20)
    private_key = models.CharField(max_length=100)
    is_active = models.BooleanField(default=True)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    class Meta(object):
        abstract = True



import functools
import json

from django import http
from django.views.decorators.csrf import csrf_exempt
from generic_request_signer.check_signature import check_signature

from request_signer import  constants, models


def signature_required(func):
    """
    Decorator to require a signed request.

    :param func:
        The view function that requires a signature.

    :returns:
        A new view function wrapped to ensure it is properly signed.
    """

    @csrf_exempt
    @functools.wraps(func)
    def _wrap(request, *args, **kwargs):
        """
        Returns bad request when:
          - no signature
          - no client
          - signature doesnt match
        """
        signature = request.GET.get(constants.SIGNATURE_PARAM_NAME)
        client_id = request.GET.get(constants.CLIENT_ID_PARAM_NAME)

        if not signature or not client_id:
            return http.HttpResponseBadRequest()

        signature_valid = False
        client = models.AuthorizedClient.get_by_client(client_id)
        if client:
            url_path = request.get_full_path()
            request_data = get_request_data(request)
            signature_valid = check_signature(signature, client.private_key, url_path, request_data)

        if signature_valid:
            return func(request, *args, **kwargs)
        else:
            return http.HttpResponseBadRequest()

    def get_request_data(request):
        if request.META.get('CONTENT_TYPE') == 'application/json':
            request_data = json.loads(request.raw_post_data)
        else:
            request_data = dict(request.POST) or None
        return request_data

    _wrap.signature_required = True
    return _wrap



from request_signer.client.generic.base import BasicSettingsApiCredentialsBackend

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

__all__ = ('DjangoSettingsApiCredentialsBackend')


class DjangoSettingsApiCredentialsBackend(BasicSettingsApiCredentialsBackend):

    def get_setting(self, name):
        client_name = self.get_client_name(name)
        setting = getattr(settings, client_name, None)
        if not setting:
            raise ImproperlyConfigured(self.CLIENT_SETTINGS_ERROR_MESSAGE.format(client_name))
        return setting

    def get_client_name(self, name):
        client_name = getattr(self.client, name, None)
        if not client_name:
            raise ImproperlyConfigured(self.CLIENT_ERROR_MESSAGE.format(name))
        return client_name





# -*- coding: utf-8 -*-
from django.test.client import Client
from request_signer.client.generic import SignedRequestFactory
from request_signer.models import AuthorizedClient
from lettuce import step

@step(u'Given a client with the client id "([^"]*)" and the private key "([^"]*)"')
def set_client_id(step, client_id, private_key):
    step.scenario.client_id = client_id
    step.scenario.private_key = private_key

@step(u'And that client is registered with the server')
def register_client(step):
    AuthorizedClient.objects.create(
        client_id=step.scenario.client_id,
        private_key=step.scenario.private_key
    )

@step(u'When the client makes a request to the server at "([^"]*)" with no signature')
def make_request(step, url_path):
    client = Client()
    step.scenario.response = client.get(url_path)

@step(u'When the client makes a request to the server at "([^"]*)" with an invalid signature')
def make_request(step, url_path):
    client = Client()
    factory = SignedRequestFactory('GET', step.scenario.client_id, step.scenario.private_key)
    url = factory.build_request_url({}, url_path)
    step.scenario.response = client.get(url + "invalidating=thesignature")

@step(u'When the client makes a request to the server at "([^"]*)" with the correct signature')
def make_request(step, url_path):
    client = Client()

    factory = SignedRequestFactory('GET', step.scenario.client_id, step.scenario.private_key)
    url = factory.build_request_url({}, url_path)
    step.scenario.response = client.get(url)

@step(u'When the client posts a request to the server at "([^"]*)" with the correct signature and the following data:')
def make_request(step, url_path):
    data = step.hashes[0]
    client = Client()
    factory = SignedRequestFactory('POST', step.scenario.client_id, step.scenario.private_key)
    url = factory.build_request_url(data, url_path)
    step.scenario.response = client.post(url, data)

@step(u'When the client posts a request to the server at "([^"]*)" with the correct signature and no data')
def make_request(step, url_path):
    data = {}
    client = Client()
    factory = SignedRequestFactory('POST', step.scenario.client_id, step.scenario.private_key)
    url = factory.build_request_url(data, url_path)
    step.scenario.response = client.post(url, data)

@step(u'Then the server should reply with a "([^"]*)"')
def assert_status_code(step, status_code):
    assert step.scenario.response.status_code == int(status_code), step.scenario.response.status_code




from setuptools import setup, find_packages

from request_signer import VERSION

setup(
    name='django-request-signer',
    version=VERSION,
    author='imtapps',
    url='https://github.com/imtapps/django-request-signer',
    description="A python library for signing http requests.",
    long_description=open('README.rst', 'r').read(),
    install_requires=file('requirements/dist.txt').read().split("\n"),
    packages=find_packages(exclude=("example", "request_signer.tests")),

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'License :: OSI Approved :: BSD License',
        'Topic :: Internet',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
